﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ModelFileManager
{
    public partial class frmMain : Form
    {
        private string _DeviationDrive = "";
        private string _BackupPath = "";

        // ----------------------------------------------------------------------------------------------------------------------

        public frmMain()
        {
            InitializeComponent();
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void frmMain_Load(object sender, EventArgs e)
        {
            // Use our location to find where the backups will go
            _BackupPath = Path.Combine(Application.StartupPath, "backups");
            if (!Directory.Exists(_BackupPath))
                Directory.CreateDirectory(_BackupPath);

            // Check for the devo right up front 
            CheckDrives();

            // watch for the drive every so often (max 1 minute)
            int checkInterval = Convert.ToInt32(ConfigurationManager.AppSettings["CheckDriveIntervalSeconds"]) * 1000;
            if (checkInterval > 60000)
                checkInterval = 60000;
            timCheckDrives.Interval = checkInterval;
            timCheckDrives.Enabled = true;

        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void timCheckDrives_Tick(object sender, EventArgs e)
        {
            CheckDrives();
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void btnBackup_Click(object sender, EventArgs e)
        {
            try
            {
                // where the current model files are
                string ModelPath = Path.Combine(_DeviationDrive, "models");

                // generate a new path under the /backups directory
                string NewBackupPath = Path.Combine(_BackupPath, DateTime.Now.ToString("yyyyMMdd-hhmmss"));
                DirectoryInfo di = Directory.CreateDirectory(NewBackupPath);

                // iterate through the files copying them
                string[] fileEntries = Directory.GetFiles(ModelPath);
                foreach (string fileName in fileEntries)
                {
                    string onlyName = Path.GetFileName(fileName);
                    string srcfile = Path.Combine(ModelPath, onlyName);
                    string dstfile = Path.Combine(NewBackupPath, onlyName);
                    File.Copy(srcfile, dstfile);
                }

                MessageBox.Show("Backup Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Backup Was NOT Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Common.WriteError("btnBackup_Click()", ex);
            }
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void btnRestore_Click(object sender, EventArgs e)
        {
            Restore frmRestore = new Restore();
            
            // pass in the paths we need
            frmRestore.BackupDirectory = _BackupPath;
            frmRestore.ModelDirectory = Path.Combine(_DeviationDrive, "models");

            // Basically just show the Restore Form
            DialogResult result = frmRestore.ShowDialog();

            // Base on the DialogResult passed back, we refresh or not
            if (result != System.Windows.Forms.DialogResult.Cancel)
                LoadModelNames();
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void btnMoveUp_Click(object sender, EventArgs e)
        {
            // Make sure *something* is selected
            if (lstModels.SelectedIndex >= 0)                
            {
                // cannot move top one up any further, but anything selected we can move up
                if (lstModels.SelectedIndex > 0)
                {
                    // get the model no from the list item text
                    int ModelNo = GetSelectedModelNo();
                    // DO it
                    MoveModel(ModelNo, true);
                    // refresh list
                    LoadModelNames();
                }
            }
            else
                MessageBox.Show("Must Select Model First", "Sorry", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void btnMoveDown_Click(object sender, EventArgs e)
        {
            // Make sure *something* is selected
            if (lstModels.SelectedIndex >= 0)
            {
                // only move down if it is not in the last slot
                if (lstModels.SelectedIndex < (lstModels.Items.Count - 1))
                {
                    // get the model no from the list item text
                    int ModelNo = GetSelectedModelNo();
                    // Do it
                    MoveModel(ModelNo, false);
                    // refresh list
                    LoadModelNames();
                }
            }
            else
                MessageBox.Show("Must Select Model First", "Sorry", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void chkShowEmpty_CheckedChanged(object sender, EventArgs e)
        {
            // just refresh the list box to pick up the new filter
            LoadModelNames();
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void btnClear_Click(object sender, EventArgs e)
        {
            // Make sure *something* is selected
            if (lstModels.SelectedIndex >= 0)
            {
                // get the model no from the list item text
                int ModelNo = GetSelectedModelNo();
                // confirm
                if (MessageBox.Show("Are you SURE you want to clear model slot #" + ModelNo.ToString() + "?", "Confirm", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation) == System.Windows.Forms.DialogResult.Yes)
                {
                    // Do it 
                    ClearModel(ModelNo);
                    // Reload the listbox
                    LoadModelNames();
                }
            }
            else
                MessageBox.Show("Must Select Model First", "Sorry", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void btnCopy_Click(object sender, EventArgs e)
        {
            // Make sure *something* is selected
            if (lstModels.SelectedIndex >= 0)
            {
                // confirm
                if (MessageBox.Show("Copy selected model to next empty model slot?", "Confirm", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation) == System.Windows.Forms.DialogResult.Yes)
                {
                    // get the model no from the list item text
                    int ModelNo = GetSelectedModelNo();
                    // Do it 
                    CopyModel(ModelNo);
                    // Reload the listbox
                    LoadModelNames();
                }
            }
            else
                MessageBox.Show("Must Select Model First", "Sorry", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void lstModels_DoubleClick(object sender, EventArgs e)
        {
            // Make sure *something* is selected
            if (lstModels.SelectedIndex >= 0)
            {
                try
                {
                    int ModelNo = GetSelectedModelNo();
                    string targetFile = Path.Combine(_DeviationDrive, "models", "model" + ModelNo.ToString() + ".ini");

                    string exeFile = ConfigurationManager.AppSettings["ModelEditor"];
                    if ((exeFile == "") || (!File.Exists(exeFile)))
                    {
                        Viewer viewer = new Viewer(targetFile);
                        viewer.Show();
                    }
                    else
                    {
                        ProcessStartInfo startInfo = new ProcessStartInfo();
                        startInfo.FileName = exeFile;
                        string argTemplate = ConfigurationManager.AppSettings["ModelEditorArgs"];
                        if (argTemplate == "")
                            argTemplate = "{0}";
                        string args = String.Format(argTemplate, "\"" + targetFile + "\"");
                        startInfo.Arguments = args;
                        Process.Start(startInfo);
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Operation Was NOT Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Common.WriteError("lstModels_DoubleClick()", ex);
                }
            }
        }


        // ----------------------------------------------------------------------------------------------------------------------

        private void btnImport_Click(object sender, EventArgs e)
        {
            // let user select the file
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.DefaultExt = ".ini";
            ofd.Filter = "Model files (.ini)|*.ini|All files (*.*)|*.*";
            DialogResult dr = ofd.ShowDialog();
            if (dr == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    string srcFile = ofd.FileName;
                    int foundModelSlot = 0;

                    // try to find an empty model slot
                    string dstFile = "";
                    for (int i = 1; i <= Common.MAX_MODELS; i++)
                    {
                        // file to test
                        string TargetFile = Path.Combine(_DeviationDrive, "models", "model" + i.ToString() + ".ini");
                        // retrive the model name.... if any
                        string ModelName = Common.GetModelName(TargetFile);
                        if (String.IsNullOrEmpty(ModelName) || ModelName == "NONE")
                        {
                            foundModelSlot = i;
                            dstFile = Path.Combine(_DeviationDrive, "models", "model" + i.ToString() + ".ini");
                            break;
                        }
                    }

                    // if we have a destination figured out... 
                    if (dstFile != "")
                    {
                        // see if we need to confirm with the user
                        bool continueFlag = false;
                        if (ConfigurationManager.AppSettings["ConfirmImport"].ToUpper().Substring(0, 1) == "Y")
                        {
                            // yes, we want to
                            if (MessageBox.Show("Are you sure you want to import \n\n" + srcFile + "\n\nto empty model slot #" + foundModelSlot.ToString() + "?", "Confirm", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation) == System.Windows.Forms.DialogResult.Yes)
                            {
                                continueFlag = true;
                            }
                        }
                        else
                            continueFlag = true;

                        // if we are to continue with the import...
                        if (continueFlag)
                        {
                            // delete the destination file
                            if (File.Exists(dstFile))
                                File.Delete(dstFile);
                            
                            string exeFile = ConfigurationManager.AppSettings["ModelConverter"];

                            // if there is no converter defined, then just copy as is
                            if (String.IsNullOrEmpty(exeFile))
                            {
                                File.Copy(srcFile, dstFile);
                                LoadModelNames();
                                MessageBox.Show("Import Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Information);
                            }
                            else
                            {
                                // if there is no path, then assume the current directory
                                if (exeFile.IndexOf(Path.PathSeparator) == -1)
                                    exeFile = Path.Combine(Path.GetDirectoryName(Application.ExecutablePath), exeFile);

                                // if the file does not exist then throw error
                                if (!File.Exists(exeFile))
                                    throw new Exception("Model Converter Utility Not Found.");
                                else
                                {
                                    DialogResult result = MessageBox.Show("Would you like to use the model converter on the import file \n\n" + srcFile + "?", "Confirm", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
                                    if (result == System.Windows.Forms.DialogResult.Yes)
                                    {
                                        ProcessStartInfo startInfo = new ProcessStartInfo();
                                        startInfo.FileName = exeFile;
                                        string argTemplate = ConfigurationManager.AppSettings["ModelConverterArgs"];
                                        if (argTemplate == "")
                                            argTemplate = "{0} {1}";
                                        string args = String.Format(argTemplate, "\"" + srcFile + "\"", "\"" + dstFile + "\"");
                                        startInfo.Arguments = args;
                                        using (Process p = Process.Start(startInfo))
                                        {
                                            p.WaitForExit();
                                        }
                                        LoadModelNames();
                                        MessageBox.Show("Import Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                    }
                                    else if (result == System.Windows.Forms.DialogResult.No)
                                    {
                                        File.Copy(srcFile, dstFile);
                                        LoadModelNames();
                                        MessageBox.Show("Import Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Information);
                                    }                                
                                }
                            }
                        }

                    }
                    else
                        MessageBox.Show("Unable to find an empty model slot.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Operation Was NOT Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Common.WriteError("btnImport_Click()", ex);
                }
            }
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void btnRename_Click(object sender, EventArgs e)
        {
            // Make sure *something* is selected
            if (lstModels.SelectedIndex >= 0)
            {
                // get the model no from the list item text
                int ModelNo = GetSelectedModelNo();

                // now get the corresponding file on the devo
                string targetFile = Path.Combine(_DeviationDrive, "models", "model" + ModelNo.ToString() + ".ini");

                // set up rename form and show user
                Rename r = new Rename(targetFile);
                System.Windows.Forms.DialogResult result = r.ShowDialog();
                if (result == System.Windows.Forms.DialogResult.OK)
                    Common.SetModelName(targetFile, r._modelName);

                // Reload the listbox
                LoadModelNames();
            }
            else
                MessageBox.Show("Must Select Model First", "Sorry", MessageBoxButtons.OK, MessageBoxIcon.Question);
        }

        // ----------------------------------------------------------------------------------------------------------------------

        private void btnExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        // ----------------------------------------------------------------------------------------------------------------------
        // Based on the selected model in the model listbox, we compute and return the model "number"
        // ----------------------------------------------------------------------------------------------------------------------
        private int GetSelectedModelNo()
        {
            int result = -1;

            try
            {
                string itemText = (string)lstModels.Items[lstModels.SelectedIndex];
                int idLength = itemText.IndexOf("-") - 1;
                string idText = itemText.Substring(0, idLength);
                result = Convert.ToInt32(idText);
            }
            catch (Exception ex)
            {
                Common.WriteError("GetSelectedModelNo()", ex);
            }

            return result;
        }

        // ----------------------------------------------------------------------------------------------------------------------
        // Routine to check all of the removeable drives for a Deviation installation.
        // ----------------------------------------------------------------------------------------------------------------------
        private void CheckDrives()
        {
            try
            {
                // if we do not know of a Deviation drive...
                if (_DeviationDrive == "")
                {
                    // scan each removable drive for the  ID file
                    foreach (var drive in DriveInfo.GetDrives())
                    {
                        if (drive.DriveType == DriveType.Removable)
                        {
                            string FilePathTest = Path.Combine(drive.RootDirectory.ToString(), "tx.ini");
                            if (File.Exists(FilePathTest))
                            {
                                // Drive found!!!
                                _DeviationDrive = drive.RootDirectory.ToString();
                                LoadModelNames();
                                break;
                            }
                        }
                    }
                }
                else
                {
                    // if we do know of a Deviation drive, check if it is still there
                    string FilePathTest = Path.Combine(_DeviationDrive, "tx.ini");
                    if (!File.Exists(FilePathTest))
                        _DeviationDrive = "";
                }

                // update the GUI based on the presence of a Devo device
                EnableGUI(_DeviationDrive != "");                
            }
            catch (Exception ex)
            {
                Common.WriteError("CheckDrives()", ex);
            }
        }

        // ----------------------------------------------------------------------------------------------------------------------
        // Utility to turn the active GUI on and off (Exit is always available)
        // ----------------------------------------------------------------------------------------------------------------------
        private void EnableGUI(bool Enable)
        {
            if (!Enable)
                lstModels.Items.Clear();

            btnBackup.Enabled = Enable;
            btnRestore.Enabled = Enable;
            btnImport.Enabled = Enable;
            btnMoveUp.Enabled = Enable;
            btnMoveDown.Enabled = Enable;
            btnCopy.Enabled = Enable;
            btnClear.Enabled = Enable;
            btnRename.Enabled = Enable;
            lstModels.Enabled = Enable;
            chkShowEmpty.Enabled = Enable;
        }

        // ----------------------------------------------------------------------------------------------------------------------
        // Refresh routine that clears and reloads the model listbox from the current Deviation install
        // ----------------------------------------------------------------------------------------------------------------------
        private void LoadModelNames()
        {
            try
            {
                // keep the model name so we can try to re-select it later
                string previousName = "";
                if (lstModels.SelectedIndex >= 0 && lstModels.SelectedIndex < lstModels.Items.Count)
                {
                    string previousSelection = (string)lstModels.Items[lstModels.SelectedIndex];
                    previousName = previousSelection.Substring(previousSelection.IndexOf("-") + 1).Trim();
                }

                // ok, now let's reload the list
                lstModels.Items.Clear();
                for (int i = 1; i < Common.MAX_MODELS; i++)
                {
                    string TargetFile = Path.Combine(_DeviationDrive, "models", "model" + i.ToString() + ".ini");
                    string ModelName = Common.GetModelName(TargetFile);
                    if (!String.IsNullOrEmpty(ModelName))
                    {
                        if (ModelName == "NONE")
                        {
                            if (chkShowEmpty.Checked)
                                lstModels.Items.Add(i.ToString() + " -");
                        }
                        else
                        {
                            lstModels.Items.Add(i.ToString() + " - " + ModelName);
                        }
                    }
                }

                // re-select previously selected model
                if (!String.IsNullOrEmpty(previousName))
                {
                    for (int i = 0; i < lstModels.Items.Count; i++)
                    {
                        if (((string)lstModels.Items[i]).IndexOf(previousName) > -1)
                        {
                            lstModels.SelectedIndex = i;
                            break;
                        }
                    }
                }

                lstModels.Focus();            
            }
            catch (Exception ex)
            {
                Common.WriteError("LoadModelNames()", ex);
            }            
        }

        // ----------------------------------------------------------------------------------------------------------------------
        // Will overwrite the specified model file with the default.ini model file
        // ----------------------------------------------------------------------------------------------------------------------
        private void ClearModel(int ModelNo)
        {
            try
            {
                string srcFile = Path.Combine(_DeviationDrive, "models", "default.ini");
                string dstFile = Path.Combine(_DeviationDrive, "models", "model" + ModelNo.ToString() + ".ini");
                File.Delete(dstFile);
                File.Copy(srcFile, dstFile);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Operation Was NOT Successful.", "Results",  MessageBoxButtons.OK, MessageBoxIcon.Error);
                Common.WriteError("ClearModel()", ex);
            }
        }

        // ----------------------------------------------------------------------------------------------------------------------
        // Will make a copy of the specified model to the next available empty slot
        // ----------------------------------------------------------------------------------------------------------------------
        private void CopyModel(int ModelNo)
        {
            try
            {
                // formulate source filespec
                string srcFile = Path.Combine(_DeviationDrive, "models", "model" + ModelNo.ToString() + ".ini");

                // try to find an empty model slot
                string dstFile = "";
                for (int i = 1; i <= Common.MAX_MODELS; i++)
                {
                    // file to test
                    string TargetFile = Path.Combine(_DeviationDrive, "models", "model" + i.ToString() + ".ini");
                    // retrive the model name.... if any
                    string ModelName = Common.GetModelName(TargetFile);
                    if (String.IsNullOrEmpty(ModelName) || ModelName == "NONE")
                    {
                        dstFile = Path.Combine(_DeviationDrive, "models", "model" + i.ToString() + ".ini");
                        break;
                    }
                }
                
                // if we have a destination figured out... fill it with the copy
                if (dstFile != "")
                {
                    if (File.Exists(dstFile))
                        File.Delete(dstFile);
                    File.Copy(srcFile, dstFile);
                }
                else
                    MessageBox.Show("Unable to find an empty model slot.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Error);

            }
            catch (Exception ex)
            {
                MessageBox.Show("Operation Was NOT Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Common.WriteError("ClearModel()", ex);
            }
        }

        // ----------------------------------------------------------------------------------------------------------------------
        // Routine that renames the model files in such a way, that the specified model is moved up or down
        // ----------------------------------------------------------------------------------------------------------------------
        private void MoveModel(int ModelNo, bool GoingUp)
        {
            try
            {
                if ((GoingUp && (ModelNo > 1)) || (!GoingUp && (ModelNo < Common.MAX_MODELS)))
                {
                    // copy the starting file to a temp name
                    string srcFile = Path.Combine(_DeviationDrive, "models", "model" + ModelNo.ToString() + ".ini");
                    string dstFile = Path.Combine(_DeviationDrive, "models", "temp.ini");
                    File.Move(srcFile, dstFile);

                    // copy the replaced file to the src file location
                    dstFile = srcFile;
                    if (GoingUp)
                        srcFile = Path.Combine(_DeviationDrive, "models", "model" + (ModelNo - 1).ToString() + ".ini");
                    else
                        srcFile = Path.Combine(_DeviationDrive, "models", "model" + (ModelNo + 1).ToString() + ".ini");
                    if (!File.Exists(srcFile))
                    {
                        string templateFile = Path.Combine(_DeviationDrive, "models", "default.ini");
                        File.Copy(templateFile, srcFile);
                    }
                    File.Move(srcFile, dstFile);
                    
                    // finally name the temp name to the replaced file
                    dstFile = srcFile;
                    srcFile = Path.Combine(_DeviationDrive, "models", "temp.ini");
                    File.Move(srcFile, dstFile);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Operation Was NOT Successful.", "Results", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Common.WriteError("MoveModel()", ex);
            }
        }

    }
}
